/* errors.c */

#include "df3util.h"


static char const * const emsgs[ec_ec_max+1] = {
  "ok",
  "bad bpv",
  "bad dimensions",
  "bad offset",
  "bad voxel",
  "bad argument",
  "not enough room on device",
  "bad filename",
  "file exists",
  "no permissions",
  "no (usable) file",
  "bad usage",
  "library/system error",
  NULL
};

char const *appLastError(Ecodes const ec) { return emsgs[(size_t)ec]; }


/* le2ae()
 * -------------------------------------------------------------------------- *
 * libdf3 error -> app error.
 */

Ecodes le2ae(df3Error const e) {
  Ecodes x = ecOk;
  switch (e) {
    case ecBadArgument:  x = ec_badArgument; break;
    case ecBadDimsAxes:  x = ec_badAxes;     break;
    case ecBadDimsBpv:   x = ec_badBPV;      break;
    case ecBadHandle:    x = ec_badArgument; break;
    case ecBadVoxAt:     x = ec_badOffset;   break;
    case ecBadVoxVal:    x = ec_badVoxel;    break;
    case ecNotDf3File:   x = ec_noFile;      break;
    case ecUnusableFile: x = ec_noFile;      break;
    case ecSysErr:       x = ec_sysError;    break;
    case ecOk:           x = ec_ok;          break;
    default:             APPPANIC;
  }
  return x;
}


/* appPanic()
 * -------------------------------------------------------------------------- *
 * cf D Lewine.
 * errno may have changed before assignment (unlikely though).
 */

void appPanic(char const * const fname, int const line) {
  int const e = errno;
  (void)fprintf(stderr, "%s:  *** aborted ***\n"
                        "\"cannot happen\" error in source '%s' line %d,\n"
                        "libc errno was %d (%s).\n",
          app_name, trimmedFn(fname), line, e, strerror(e));
  abort();
}

/* ------------------------------------------------------------------ *
 *  this file is part of 'df3util', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.     *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.               *
 *  all rights reserved.                                              *
 * ------------------------------------------------------------------ */
